#define _CFFI_

/* We try to define Py_LIMITED_API before including Python.h.

   Mess: we can only define it if Py_DEBUG, Py_TRACE_REFS and
   Py_REF_DEBUG are not defined.  This is a best-effort approximation:
   we can learn about Py_DEBUG from pyconfig.h, but it is unclear if
   the same works for the other two macros.  Py_DEBUG implies them,
   but not the other way around.

   The implementation is messy (issue #350): on Windows, with _MSC_VER,
   we have to define Py_LIMITED_API even before including pyconfig.h.
   In that case, we guess what pyconfig.h will do to the macros above,
   and check our guess after the #include.

   Note that on Windows, with CPython 3.x, you need >= 3.5 and virtualenv
   version >= 16.0.0.  With older versions of either, you don't get a
   copy of PYTHON3.DLL in the virtualenv.  We can't check the version of
   CPython *before* we even include pyconfig.h.  ffi.set_source() puts
   a ``#define _CFFI_NO_LIMITED_API'' at the start of this file if it is
   running on Windows < 3.5, as an attempt at fixing it, but that's
   arguably wrong because it may not be the target version of Python.
   Still better than nothing I guess.  As another workaround, you can
   remove the definition of Py_LIMITED_API here.

   See also 'py_limited_api' in cffi/setuptools_ext.py.
*/
#if !defined(_CFFI_USE_EMBEDDING) && !defined(Py_LIMITED_API)
#  ifdef _MSC_VER
#    if !defined(_DEBUG) && !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#    include <pyconfig.h>
     /* sanity-check: Py_LIMITED_API will cause crashes if any of these
        are also defined.  Normally, the Python file PC/pyconfig.h does not
        cause any of these to be defined, with the exception that _DEBUG
        causes Py_DEBUG.  Double-check that. */
#    ifdef Py_LIMITED_API
#      if defined(Py_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_DEBUG, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_TRACE_REFS)
#        error "pyconfig.h unexpectedly defines Py_TRACE_REFS, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_REF_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_REF_DEBUG, but Py_LIMITED_API is set"
#      endif
#    endif
#  else
#    include <pyconfig.h>
#    if !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#  endif
#endif

#include <Python.h>
#ifdef __cplusplus
extern "C" {
#endif
#include <stddef.h>

/* This part is from file 'cffi/parse_c_type.h'.  It is copied at the
   beginning of C sources generated by CFFI's ffi.set_source(). */

typedef void *_cffi_opcode_t;

#define _CFFI_OP(opcode, arg)   (_cffi_opcode_t)(opcode | (((uintptr_t)(arg)) << 8))
#define _CFFI_GETOP(cffi_opcode)    ((unsigned char)(uintptr_t)cffi_opcode)
#define _CFFI_GETARG(cffi_opcode)   (((intptr_t)cffi_opcode) >> 8)

#define _CFFI_OP_PRIMITIVE       1
#define _CFFI_OP_POINTER         3
#define _CFFI_OP_ARRAY           5
#define _CFFI_OP_OPEN_ARRAY      7
#define _CFFI_OP_STRUCT_UNION    9
#define _CFFI_OP_ENUM           11
#define _CFFI_OP_FUNCTION       13
#define _CFFI_OP_FUNCTION_END   15
#define _CFFI_OP_NOOP           17
#define _CFFI_OP_BITFIELD       19
#define _CFFI_OP_TYPENAME       21
#define _CFFI_OP_CPYTHON_BLTN_V 23   // varargs
#define _CFFI_OP_CPYTHON_BLTN_N 25   // noargs
#define _CFFI_OP_CPYTHON_BLTN_O 27   // O  (i.e. a single arg)
#define _CFFI_OP_CONSTANT       29
#define _CFFI_OP_CONSTANT_INT   31
#define _CFFI_OP_GLOBAL_VAR     33
#define _CFFI_OP_DLOPEN_FUNC    35
#define _CFFI_OP_DLOPEN_CONST   37
#define _CFFI_OP_GLOBAL_VAR_F   39
#define _CFFI_OP_EXTERN_PYTHON  41

#define _CFFI_PRIM_VOID          0
#define _CFFI_PRIM_BOOL          1
#define _CFFI_PRIM_CHAR          2
#define _CFFI_PRIM_SCHAR         3
#define _CFFI_PRIM_UCHAR         4
#define _CFFI_PRIM_SHORT         5
#define _CFFI_PRIM_USHORT        6
#define _CFFI_PRIM_INT           7
#define _CFFI_PRIM_UINT          8
#define _CFFI_PRIM_LONG          9
#define _CFFI_PRIM_ULONG        10
#define _CFFI_PRIM_LONGLONG     11
#define _CFFI_PRIM_ULONGLONG    12
#define _CFFI_PRIM_FLOAT        13
#define _CFFI_PRIM_DOUBLE       14
#define _CFFI_PRIM_LONGDOUBLE   15

#define _CFFI_PRIM_WCHAR        16
#define _CFFI_PRIM_INT8         17
#define _CFFI_PRIM_UINT8        18
#define _CFFI_PRIM_INT16        19
#define _CFFI_PRIM_UINT16       20
#define _CFFI_PRIM_INT32        21
#define _CFFI_PRIM_UINT32       22
#define _CFFI_PRIM_INT64        23
#define _CFFI_PRIM_UINT64       24
#define _CFFI_PRIM_INTPTR       25
#define _CFFI_PRIM_UINTPTR      26
#define _CFFI_PRIM_PTRDIFF      27
#define _CFFI_PRIM_SIZE         28
#define _CFFI_PRIM_SSIZE        29
#define _CFFI_PRIM_INT_LEAST8   30
#define _CFFI_PRIM_UINT_LEAST8  31
#define _CFFI_PRIM_INT_LEAST16  32
#define _CFFI_PRIM_UINT_LEAST16 33
#define _CFFI_PRIM_INT_LEAST32  34
#define _CFFI_PRIM_UINT_LEAST32 35
#define _CFFI_PRIM_INT_LEAST64  36
#define _CFFI_PRIM_UINT_LEAST64 37
#define _CFFI_PRIM_INT_FAST8    38
#define _CFFI_PRIM_UINT_FAST8   39
#define _CFFI_PRIM_INT_FAST16   40
#define _CFFI_PRIM_UINT_FAST16  41
#define _CFFI_PRIM_INT_FAST32   42
#define _CFFI_PRIM_UINT_FAST32  43
#define _CFFI_PRIM_INT_FAST64   44
#define _CFFI_PRIM_UINT_FAST64  45
#define _CFFI_PRIM_INTMAX       46
#define _CFFI_PRIM_UINTMAX      47
#define _CFFI_PRIM_FLOATCOMPLEX 48
#define _CFFI_PRIM_DOUBLECOMPLEX 49
#define _CFFI_PRIM_CHAR16       50
#define _CFFI_PRIM_CHAR32       51

#define _CFFI__NUM_PRIM         52
#define _CFFI__UNKNOWN_PRIM           (-1)
#define _CFFI__UNKNOWN_FLOAT_PRIM     (-2)
#define _CFFI__UNKNOWN_LONG_DOUBLE    (-3)

#define _CFFI__IO_FILE_STRUCT         (-1)


struct _cffi_global_s {
    const char *name;
    void *address;
    _cffi_opcode_t type_op;
    void *size_or_direct_fn;  // OP_GLOBAL_VAR: size, or 0 if unknown
                              // OP_CPYTHON_BLTN_*: addr of direct function
};

struct _cffi_getconst_s {
    unsigned long long value;
    const struct _cffi_type_context_s *ctx;
    int gindex;
};

struct _cffi_struct_union_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_STRUCT_UNION
    int flags;               // _CFFI_F_* flags below
    size_t size;
    int alignment;
    int first_field_index;   // -> _cffi_fields array
    int num_fields;
};
#define _CFFI_F_UNION         0x01   // is a union, not a struct
#define _CFFI_F_CHECK_FIELDS  0x02   // complain if fields are not in the
                                     // "standard layout" or if some are missing
#define _CFFI_F_PACKED        0x04   // for CHECK_FIELDS, assume a packed struct
#define _CFFI_F_EXTERNAL      0x08   // in some other ffi.include()
#define _CFFI_F_OPAQUE        0x10   // opaque

struct _cffi_field_s {
    const char *name;
    size_t field_offset;
    size_t field_size;
    _cffi_opcode_t field_type_op;
};

struct _cffi_enum_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_ENUM
    int type_prim;           // _CFFI_PRIM_xxx
    const char *enumerators; // comma-delimited string
};

struct _cffi_typename_s {
    const char *name;
    int type_index;   /* if opaque, points to a possibly artificial
                         OP_STRUCT which is itself opaque */
};

struct _cffi_type_context_s {
    _cffi_opcode_t *types;
    const struct _cffi_global_s *globals;
    const struct _cffi_field_s *fields;
    const struct _cffi_struct_union_s *struct_unions;
    const struct _cffi_enum_s *enums;
    const struct _cffi_typename_s *typenames;
    int num_globals;
    int num_struct_unions;
    int num_enums;
    int num_typenames;
    const char *const *includes;
    int num_types;
    int flags;      /* future extension */
};

struct _cffi_parse_info_s {
    const struct _cffi_type_context_s *ctx;
    _cffi_opcode_t *output;
    unsigned int output_size;
    size_t error_location;
    const char *error_message;
};

struct _cffi_externpy_s {
    const char *name;
    size_t size_of_result;
    void *reserved1, *reserved2;
};

#ifdef _CFFI_INTERNAL
static int parse_c_type(struct _cffi_parse_info_s *info, const char *input);
static int search_in_globals(const struct _cffi_type_context_s *ctx,
                             const char *search, size_t search_len);
static int search_in_struct_unions(const struct _cffi_type_context_s *ctx,
                                   const char *search, size_t search_len);
#endif

/* this block of #ifs should be kept exactly identical between
   c/_cffi_backend.c, cffi/vengine_cpy.py, cffi/vengine_gen.py
   and cffi/_cffi_include.h */
#if defined(_MSC_VER)
# include <malloc.h>   /* for alloca() */
# if _MSC_VER < 1600   /* MSVC < 2010 */
   typedef __int8 int8_t;
   typedef __int16 int16_t;
   typedef __int32 int32_t;
   typedef __int64 int64_t;
   typedef unsigned __int8 uint8_t;
   typedef unsigned __int16 uint16_t;
   typedef unsigned __int32 uint32_t;
   typedef unsigned __int64 uint64_t;
   typedef __int8 int_least8_t;
   typedef __int16 int_least16_t;
   typedef __int32 int_least32_t;
   typedef __int64 int_least64_t;
   typedef unsigned __int8 uint_least8_t;
   typedef unsigned __int16 uint_least16_t;
   typedef unsigned __int32 uint_least32_t;
   typedef unsigned __int64 uint_least64_t;
   typedef __int8 int_fast8_t;
   typedef __int16 int_fast16_t;
   typedef __int32 int_fast32_t;
   typedef __int64 int_fast64_t;
   typedef unsigned __int8 uint_fast8_t;
   typedef unsigned __int16 uint_fast16_t;
   typedef unsigned __int32 uint_fast32_t;
   typedef unsigned __int64 uint_fast64_t;
   typedef __int64 intmax_t;
   typedef unsigned __int64 uintmax_t;
# else
#  include <stdint.h>
# endif
# if _MSC_VER < 1800   /* MSVC < 2013 */
#  ifndef __cplusplus
    typedef unsigned char _Bool;
#  endif
# endif
# define _cffi_float_complex_t   _Fcomplex    /* include <complex.h> for it */
# define _cffi_double_complex_t  _Dcomplex    /* include <complex.h> for it */
#else
# include <stdint.h>
# if (defined (__SVR4) && defined (__sun)) || defined(_AIX) || defined(__hpux)
#  include <alloca.h>
# endif
# define _cffi_float_complex_t   float _Complex
# define _cffi_double_complex_t  double _Complex
#endif

#ifdef __GNUC__
# define _CFFI_UNUSED_FN  __attribute__((unused))
#else
# define _CFFI_UNUSED_FN  /* nothing */
#endif

#ifdef __cplusplus
# ifndef _Bool
   typedef bool _Bool;   /* semi-hackish: C++ has no _Bool; bool is builtin */
# endif
#endif

/**********  CPython-specific section  **********/
#ifndef PYPY_VERSION


#if PY_MAJOR_VERSION >= 3
# define PyInt_FromLong PyLong_FromLong
#endif

#define _cffi_from_c_double PyFloat_FromDouble
#define _cffi_from_c_float PyFloat_FromDouble
#define _cffi_from_c_long PyInt_FromLong
#define _cffi_from_c_ulong PyLong_FromUnsignedLong
#define _cffi_from_c_longlong PyLong_FromLongLong
#define _cffi_from_c_ulonglong PyLong_FromUnsignedLongLong
#define _cffi_from_c__Bool PyBool_FromLong

#define _cffi_to_c_double PyFloat_AsDouble
#define _cffi_to_c_float PyFloat_AsDouble

#define _cffi_from_c_int(x, type)                                        \
    (((type)-1) > 0 ? /* unsigned */                                     \
        (sizeof(type) < sizeof(long) ?                                   \
            PyInt_FromLong((long)x) :                                    \
         sizeof(type) == sizeof(long) ?                                  \
            PyLong_FromUnsignedLong((unsigned long)x) :                  \
            PyLong_FromUnsignedLongLong((unsigned long long)x)) :        \
        (sizeof(type) <= sizeof(long) ?                                  \
            PyInt_FromLong((long)x) :                                    \
            PyLong_FromLongLong((long long)x)))

#define _cffi_to_c_int(o, type)                                          \
    ((type)(                                                             \
     sizeof(type) == 1 ? (((type)-1) > 0 ? (type)_cffi_to_c_u8(o)        \
                                         : (type)_cffi_to_c_i8(o)) :     \
     sizeof(type) == 2 ? (((type)-1) > 0 ? (type)_cffi_to_c_u16(o)       \
                                         : (type)_cffi_to_c_i16(o)) :    \
     sizeof(type) == 4 ? (((type)-1) > 0 ? (type)_cffi_to_c_u32(o)       \
                                         : (type)_cffi_to_c_i32(o)) :    \
     sizeof(type) == 8 ? (((type)-1) > 0 ? (type)_cffi_to_c_u64(o)       \
                                         : (type)_cffi_to_c_i64(o)) :    \
     (Py_FatalError("unsupported size for type " #type), (type)0)))

#define _cffi_to_c_i8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[1])
#define _cffi_to_c_u8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[2])
#define _cffi_to_c_i16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[3])
#define _cffi_to_c_u16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[4])
#define _cffi_to_c_i32                                                   \
                 ((int(*)(PyObject *))_cffi_exports[5])
#define _cffi_to_c_u32                                                   \
                 ((unsigned int(*)(PyObject *))_cffi_exports[6])
#define _cffi_to_c_i64                                                   \
                 ((long long(*)(PyObject *))_cffi_exports[7])
#define _cffi_to_c_u64                                                   \
                 ((unsigned long long(*)(PyObject *))_cffi_exports[8])
#define _cffi_to_c_char                                                  \
                 ((int(*)(PyObject *))_cffi_exports[9])
#define _cffi_from_c_pointer                                             \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[10])
#define _cffi_to_c_pointer                                               \
    ((char *(*)(PyObject *, struct _cffi_ctypedescr *))_cffi_exports[11])
#define _cffi_get_struct_layout                                          \
    not used any more
#define _cffi_restore_errno                                              \
    ((void(*)(void))_cffi_exports[13])
#define _cffi_save_errno                                                 \
    ((void(*)(void))_cffi_exports[14])
#define _cffi_from_c_char                                                \
    ((PyObject *(*)(char))_cffi_exports[15])
#define _cffi_from_c_deref                                               \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[16])
#define _cffi_to_c                                                       \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[17])
#define _cffi_from_c_struct                                              \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[18])
#define _cffi_to_c_wchar_t                                               \
    ((_cffi_wchar_t(*)(PyObject *))_cffi_exports[19])
#define _cffi_from_c_wchar_t                                             \
    ((PyObject *(*)(_cffi_wchar_t))_cffi_exports[20])
#define _cffi_to_c_long_double                                           \
    ((long double(*)(PyObject *))_cffi_exports[21])
#define _cffi_to_c__Bool                                                 \
    ((_Bool(*)(PyObject *))_cffi_exports[22])
#define _cffi_prepare_pointer_call_argument                              \
    ((Py_ssize_t(*)(struct _cffi_ctypedescr *,                           \
                    PyObject *, char **))_cffi_exports[23])
#define _cffi_convert_array_from_object                                  \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[24])
#define _CFFI_CPIDX  25
#define _cffi_call_python                                                \
    ((void(*)(struct _cffi_externpy_s *, char *))_cffi_exports[_CFFI_CPIDX])
#define _cffi_to_c_wchar3216_t                                           \
    ((int(*)(PyObject *))_cffi_exports[26])
#define _cffi_from_c_wchar3216_t                                         \
    ((PyObject *(*)(int))_cffi_exports[27])
#define _CFFI_NUM_EXPORTS 28

struct _cffi_ctypedescr;

static void *_cffi_exports[_CFFI_NUM_EXPORTS];

#define _cffi_type(index)   (                           \
    assert((((uintptr_t)_cffi_types[index]) & 1) == 0), \
    (struct _cffi_ctypedescr *)_cffi_types[index])

static PyObject *_cffi_init(const char *module_name, Py_ssize_t version,
                            const struct _cffi_type_context_s *ctx)
{
    PyObject *module, *o_arg, *new_module;
    void *raw[] = {
        (void *)module_name,
        (void *)version,
        (void *)_cffi_exports,
        (void *)ctx,
    };

    module = PyImport_ImportModule("_cffi_backend");
    if (module == NULL)
        goto failure;

    o_arg = PyLong_FromVoidPtr((void *)raw);
    if (o_arg == NULL)
        goto failure;

    new_module = PyObject_CallMethod(
        module, (char *)"_init_cffi_1_0_external_module", (char *)"O", o_arg);

    Py_DECREF(o_arg);
    Py_DECREF(module);
    return new_module;

  failure:
    Py_XDECREF(module);
    return NULL;
}


#ifdef HAVE_WCHAR_H
typedef wchar_t _cffi_wchar_t;
#else
typedef uint16_t _cffi_wchar_t;   /* same random pick as _cffi_backend.c */
#endif

_CFFI_UNUSED_FN static uint16_t _cffi_to_c_char16_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return (uint16_t)_cffi_to_c_wchar_t(o);
    else
        return (uint16_t)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char16_t(uint16_t x)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

_CFFI_UNUSED_FN static int _cffi_to_c_char32_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return (int)_cffi_to_c_wchar_t(o);
    else
        return (int)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char32_t(unsigned int x)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

union _cffi_union_alignment_u {
    unsigned char m_char;
    unsigned short m_short;
    unsigned int m_int;
    unsigned long m_long;
    unsigned long long m_longlong;
    float m_float;
    double m_double;
    long double m_longdouble;
};

struct _cffi_freeme_s {
    struct _cffi_freeme_s *next;
    union _cffi_union_alignment_u alignment;
};

_CFFI_UNUSED_FN static int
_cffi_convert_array_argument(struct _cffi_ctypedescr *ctptr, PyObject *arg,
                             char **output_data, Py_ssize_t datasize,
                             struct _cffi_freeme_s **freeme)
{
    char *p;
    if (datasize < 0)
        return -1;

    p = *output_data;
    if (p == NULL) {
        struct _cffi_freeme_s *fp = (struct _cffi_freeme_s *)PyObject_Malloc(
            offsetof(struct _cffi_freeme_s, alignment) + (size_t)datasize);
        if (fp == NULL)
            return -1;
        fp->next = *freeme;
        *freeme = fp;
        p = *output_data = (char *)&fp->alignment;
    }
    memset((void *)p, 0, (size_t)datasize);
    return _cffi_convert_array_from_object(p, ctptr, arg);
}

_CFFI_UNUSED_FN static void
_cffi_free_array_arguments(struct _cffi_freeme_s *freeme)
{
    do {
        void *p = (void *)freeme;
        freeme = freeme->next;
        PyObject_Free(p);
    } while (freeme != NULL);
}

/**********  end CPython-specific section  **********/
#else
_CFFI_UNUSED_FN
static void (*_cffi_call_python_org)(struct _cffi_externpy_s *, char *);
# define _cffi_call_python  _cffi_call_python_org
#endif


#define _cffi_array_len(array)   (sizeof(array) / sizeof((array)[0]))

#define _cffi_prim_int(size, sign)                                      \
    ((size) == 1 ? ((sign) ? _CFFI_PRIM_INT8  : _CFFI_PRIM_UINT8)  :    \
     (size) == 2 ? ((sign) ? _CFFI_PRIM_INT16 : _CFFI_PRIM_UINT16) :    \
     (size) == 4 ? ((sign) ? _CFFI_PRIM_INT32 : _CFFI_PRIM_UINT32) :    \
     (size) == 8 ? ((sign) ? _CFFI_PRIM_INT64 : _CFFI_PRIM_UINT64) :    \
     _CFFI__UNKNOWN_PRIM)

#define _cffi_prim_float(size)                                          \
    ((size) == sizeof(float) ? _CFFI_PRIM_FLOAT :                       \
     (size) == sizeof(double) ? _CFFI_PRIM_DOUBLE :                     \
     (size) == sizeof(long double) ? _CFFI__UNKNOWN_LONG_DOUBLE :       \
     _CFFI__UNKNOWN_FLOAT_PRIM)

#define _cffi_check_int(got, got_nonpos, expected)      \
    ((got_nonpos) == (expected <= 0) &&                 \
     (got) == (unsigned long long)expected)

#ifdef MS_WIN32
# define _cffi_stdcall  __stdcall
#else
# define _cffi_stdcall  /* nothing */
#endif

#ifdef __cplusplus
}
#endif

/************************************************************/


     #include "ADIDatCAPI_mex.h"   // the C header of the library


/************************************************************/

static void *_cffi_types[] = {
/*  0 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // ADIResultCode()(ADIResultCode, wchar_t *, long, long *)
/*  1 */ _CFFI_OP(_CFFI_OP_ENUM, 2), // ADIResultCode
/*  2 */ _CFFI_OP(_CFFI_OP_POINTER, 92), // wchar_t *
/*  3 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 9), // long
/*  4 */ _CFFI_OP(_CFFI_OP_POINTER, 3), // long *
/*  5 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  6 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // ADIResultCode()(struct ADI_CommentsHandle__ * *)
/*  7 */ _CFFI_OP(_CFFI_OP_POINTER, 10), // struct ADI_CommentsHandle__ * *
/*  8 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  9 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // ADIResultCode()(struct ADI_CommentsHandle__ *)
/* 10 */ _CFFI_OP(_CFFI_OP_POINTER, 90), // struct ADI_CommentsHandle__ *
/* 11 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 12 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // ADIResultCode()(struct ADI_CommentsHandle__ *, long *, long *, long *, wchar_t *, long, long *)
/* 13 */ _CFFI_OP(_CFFI_OP_NOOP, 10),
/* 14 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 15 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 16 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 17 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 18 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 9),
/* 19 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 20 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 21 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // ADIResultCode()(struct ADI_FileHandle__ * *)
/* 22 */ _CFFI_OP(_CFFI_OP_POINTER, 25), // struct ADI_FileHandle__ * *
/* 23 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 24 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // ADIResultCode()(struct ADI_FileHandle__ *, long *)
/* 25 */ _CFFI_OP(_CFFI_OP_POINTER, 91), // struct ADI_FileHandle__ *
/* 26 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 27 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 28 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // ADIResultCode()(struct ADI_FileHandle__ *, long, long *)
/* 29 */ _CFFI_OP(_CFFI_OP_NOOP, 25),
/* 30 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 9),
/* 31 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 32 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 33 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // ADIResultCode()(struct ADI_FileHandle__ *, long, long *, double *, long *)
/* 34 */ _CFFI_OP(_CFFI_OP_NOOP, 25),
/* 35 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 9),
/* 36 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 37 */ _CFFI_OP(_CFFI_OP_POINTER, 87), // double *
/* 38 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 39 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 40 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // ADIResultCode()(struct ADI_FileHandle__ *, long, long, double *)
/* 41 */ _CFFI_OP(_CFFI_OP_NOOP, 25),
/* 42 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 9),
/* 43 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 9),
/* 44 */ _CFFI_OP(_CFFI_OP_NOOP, 37),
/* 45 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 46 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // ADIResultCode()(struct ADI_FileHandle__ *, long, long, long *)
/* 47 */ _CFFI_OP(_CFFI_OP_NOOP, 25),
/* 48 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 9),
/* 49 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 9),
/* 50 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 51 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 52 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // ADIResultCode()(struct ADI_FileHandle__ *, long, long, long, ADICDataFlags, long, float *, long *)
/* 53 */ _CFFI_OP(_CFFI_OP_NOOP, 25),
/* 54 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 9),
/* 55 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 9),
/* 56 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 9),
/* 57 */ _CFFI_OP(_CFFI_OP_ENUM, 0), // ADICDataFlags
/* 58 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 9),
/* 59 */ _CFFI_OP(_CFFI_OP_POINTER, 88), // float *
/* 60 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 61 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 62 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // ADIResultCode()(struct ADI_FileHandle__ *, long, long, wchar_t *, long, long *)
/* 63 */ _CFFI_OP(_CFFI_OP_NOOP, 25),
/* 64 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 9),
/* 65 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 9),
/* 66 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 67 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 9),
/* 68 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 69 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 70 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // ADIResultCode()(struct ADI_FileHandle__ *, long, struct ADI_CommentsHandle__ * *)
/* 71 */ _CFFI_OP(_CFFI_OP_NOOP, 25),
/* 72 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 9),
/* 73 */ _CFFI_OP(_CFFI_OP_NOOP, 7),
/* 74 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 75 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // ADIResultCode()(struct ADI_FileHandle__ *, long, wchar_t *, long, long *)
/* 76 */ _CFFI_OP(_CFFI_OP_NOOP, 25),
/* 77 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 9),
/* 78 */ _CFFI_OP(_CFFI_OP_NOOP, 2),
/* 79 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 9),
/* 80 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 81 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 82 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // ADIResultCode()(wchar_t const *, struct ADI_FileHandle__ * *, ADIFileOpenMode)
/* 83 */ _CFFI_OP(_CFFI_OP_POINTER, 92), // wchar_t const *
/* 84 */ _CFFI_OP(_CFFI_OP_NOOP, 22),
/* 85 */ _CFFI_OP(_CFFI_OP_ENUM, 1), // ADIFileOpenMode
/* 86 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 87 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14), // double
/* 88 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13), // float
/* 89 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7), // int
/* 90 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 0), // struct ADI_CommentsHandle__
/* 91 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 1), // struct ADI_FileHandle__
/* 92 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 16), // wchar_t
};

static int _cffi_const_kADICDataAtSampleRate(unsigned long long *o)
{
  int n = (kADICDataAtSampleRate) <= 0;
  *o = (unsigned long long)((kADICDataAtSampleRate) | 0);  /* check that kADICDataAtSampleRate is an integer */
  return n;
}

static int _cffi_const_kADICDataAtTickRate(unsigned long long *o)
{
  int n = (kADICDataAtTickRate) <= 0;
  *o = (unsigned long long)((kADICDataAtTickRate) | 0);  /* check that kADICDataAtTickRate is an integer */
  return n;
}

static int _cffi_const_kOpenFileForReadOnly(unsigned long long *o)
{
  int n = (kOpenFileForReadOnly) <= 0;
  *o = (unsigned long long)((kOpenFileForReadOnly) | 0);  /* check that kOpenFileForReadOnly is an integer */
  return n;
}

static int _cffi_const_kOpenFileForReadAndWrite(unsigned long long *o)
{
  int n = (kOpenFileForReadAndWrite) <= 0;
  *o = (unsigned long long)((kOpenFileForReadAndWrite) | 0);  /* check that kOpenFileForReadAndWrite is an integer */
  return n;
}

static int _cffi_const_kResultSuccess(unsigned long long *o)
{
  int n = (kResultSuccess) <= 0;
  *o = (unsigned long long)((kResultSuccess) | 0);  /* check that kResultSuccess is an integer */
  return n;
}

static int _cffi_const_kResultErrorFlagBit(unsigned long long *o)
{
  int n = (kResultErrorFlagBit) <= 0;
  *o = (unsigned long long)((kResultErrorFlagBit) | 0);  /* check that kResultErrorFlagBit is an integer */
  return n;
}

static int _cffi_const_kResultInvalidArg(unsigned long long *o)
{
  int n = (kResultInvalidArg) <= 0;
  *o = (unsigned long long)((kResultInvalidArg) | 0);  /* check that kResultInvalidArg is an integer */
  return n;
}

static int _cffi_const_kResultFail(unsigned long long *o)
{
  int n = (kResultFail) <= 0;
  *o = (unsigned long long)((kResultFail) | 0);  /* check that kResultFail is an integer */
  return n;
}

static int _cffi_const_kResultFileNotFound(unsigned long long *o)
{
  int n = (kResultFileNotFound) <= 0;
  *o = (unsigned long long)((kResultFileNotFound) | 0);  /* check that kResultFileNotFound is an integer */
  return n;
}

static int _cffi_const_kResultADICAPIMsgBase(unsigned long long *o)
{
  int n = (kResultADICAPIMsgBase) <= 0;
  *o = (unsigned long long)((kResultADICAPIMsgBase) | 0);  /* check that kResultADICAPIMsgBase is an integer */
  return n;
}

static int _cffi_const_kResultFileIOError(unsigned long long *o)
{
  int n = (kResultFileIOError) <= 0;
  *o = (unsigned long long)((kResultFileIOError) | 0);  /* check that kResultFileIOError is an integer */
  return n;
}

static int _cffi_const_kResultFileOpenFail(unsigned long long *o)
{
  int n = (kResultFileOpenFail) <= 0;
  *o = (unsigned long long)((kResultFileOpenFail) | 0);  /* check that kResultFileOpenFail is an integer */
  return n;
}

static int _cffi_const_kResultInvalidFileHandle(unsigned long long *o)
{
  int n = (kResultInvalidFileHandle) <= 0;
  *o = (unsigned long long)((kResultInvalidFileHandle) | 0);  /* check that kResultInvalidFileHandle is an integer */
  return n;
}

static int _cffi_const_kResultInvalidPosition(unsigned long long *o)
{
  int n = (kResultInvalidPosition) <= 0;
  *o = (unsigned long long)((kResultInvalidPosition) | 0);  /* check that kResultInvalidPosition is an integer */
  return n;
}

static int _cffi_const_kResultInvalidCommentNum(unsigned long long *o)
{
  int n = (kResultInvalidCommentNum) <= 0;
  *o = (unsigned long long)((kResultInvalidCommentNum) | 0);  /* check that kResultInvalidCommentNum is an integer */
  return n;
}

static int _cffi_const_kResultNoData(unsigned long long *o)
{
  int n = (kResultNoData) <= 0;
  *o = (unsigned long long)((kResultNoData) | 0);  /* check that kResultNoData is an integer */
  return n;
}

static int _cffi_const_kResultBufferTooSmall(unsigned long long *o)
{
  int n = (kResultBufferTooSmall) <= 0;
  *o = (unsigned long long)((kResultBufferTooSmall) | 0);  /* check that kResultBufferTooSmall is an integer */
  return n;
}

static ADIResultCode _cffi_d_ADI_CloseCommentsAccessor(struct ADI_CommentsHandle__ * * x0)
{
  return ADI_CloseCommentsAccessor(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ADI_CloseCommentsAccessor(PyObject *self, PyObject *arg0)
{
  struct ADI_CommentsHandle__ * * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  ADIResultCode result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(7), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ADI_CommentsHandle__ * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(7), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ADI_CloseCommentsAccessor(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ADI_CloseCommentsAccessor _cffi_d_ADI_CloseCommentsAccessor
#endif

static ADIResultCode _cffi_d_ADI_CloseFile(struct ADI_FileHandle__ * * x0)
{
  return ADI_CloseFile(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ADI_CloseFile(PyObject *self, PyObject *arg0)
{
  struct ADI_FileHandle__ * * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  ADIResultCode result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ADI_FileHandle__ * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ADI_CloseFile(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ADI_CloseFile _cffi_d_ADI_CloseFile
#endif

static ADIResultCode _cffi_d_ADI_CreateCommentsAccessor(struct ADI_FileHandle__ * x0, long x1, struct ADI_CommentsHandle__ * * x2)
{
  return ADI_CreateCommentsAccessor(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ADI_CreateCommentsAccessor(PyObject *self, PyObject *args)
{
  struct ADI_FileHandle__ * x0;
  long x1;
  struct ADI_CommentsHandle__ * * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  ADIResultCode result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ADI_CreateCommentsAccessor", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(25), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ADI_FileHandle__ *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(25), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, long);
  if (x1 == (long)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(7), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (struct ADI_CommentsHandle__ * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(7), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ADI_CreateCommentsAccessor(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ADI_CreateCommentsAccessor _cffi_d_ADI_CreateCommentsAccessor
#endif

static ADIResultCode _cffi_d_ADI_GetChannelName(struct ADI_FileHandle__ * x0, long x1, wchar_t * x2, long x3, long * x4)
{
  return ADI_GetChannelName(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ADI_GetChannelName(PyObject *self, PyObject *args)
{
  struct ADI_FileHandle__ * x0;
  long x1;
  wchar_t * x2;
  long x3;
  long * x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  ADIResultCode result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "ADI_GetChannelName", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(25), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ADI_FileHandle__ *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(25), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, long);
  if (x1 == (long)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (wchar_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, long);
  if (x3 == (long)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (long *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ADI_GetChannelName(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ADI_GetChannelName _cffi_d_ADI_GetChannelName
#endif

static ADIResultCode _cffi_d_ADI_GetCommentInfo(struct ADI_CommentsHandle__ * x0, long * x1, long * x2, long * x3, wchar_t * x4, long x5, long * x6)
{
  return ADI_GetCommentInfo(x0, x1, x2, x3, x4, x5, x6);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ADI_GetCommentInfo(PyObject *self, PyObject *args)
{
  struct ADI_CommentsHandle__ * x0;
  long * x1;
  long * x2;
  long * x3;
  wchar_t * x4;
  long x5;
  long * x6;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  ADIResultCode result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;

  if (!PyArg_UnpackTuple(args, "ADI_GetCommentInfo", 7, 7, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(10), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ADI_CommentsHandle__ *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(10), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (long *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (long *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (long *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (wchar_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x5 = _cffi_to_c_int(arg5, long);
  if (x5 == (long)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg6, (char **)&x6);
  if (datasize != 0) {
    x6 = ((size_t)datasize) <= 640 ? (long *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg6, (char **)&x6,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ADI_GetCommentInfo(x0, x1, x2, x3, x4, x5, x6); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ADI_GetCommentInfo _cffi_d_ADI_GetCommentInfo
#endif

static ADIResultCode _cffi_d_ADI_GetErrorMessage(ADIResultCode x0, wchar_t * x1, long x2, long * x3)
{
  return ADI_GetErrorMessage(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ADI_GetErrorMessage(PyObject *self, PyObject *args)
{
  ADIResultCode x0;
  wchar_t * x1;
  long x2;
  long * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  ADIResultCode result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ADI_GetErrorMessage", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(1), arg0) < 0)
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (wchar_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, long);
  if (x2 == (long)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (long *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ADI_GetErrorMessage(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ADI_GetErrorMessage _cffi_d_ADI_GetErrorMessage
#endif

static ADIResultCode _cffi_d_ADI_GetNumSamplesInRecord(struct ADI_FileHandle__ * x0, long x1, long x2, long * x3)
{
  return ADI_GetNumSamplesInRecord(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ADI_GetNumSamplesInRecord(PyObject *self, PyObject *args)
{
  struct ADI_FileHandle__ * x0;
  long x1;
  long x2;
  long * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  ADIResultCode result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ADI_GetNumSamplesInRecord", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(25), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ADI_FileHandle__ *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(25), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, long);
  if (x1 == (long)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, long);
  if (x2 == (long)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (long *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ADI_GetNumSamplesInRecord(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ADI_GetNumSamplesInRecord _cffi_d_ADI_GetNumSamplesInRecord
#endif

static ADIResultCode _cffi_d_ADI_GetNumTicksInRecord(struct ADI_FileHandle__ * x0, long x1, long * x2)
{
  return ADI_GetNumTicksInRecord(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ADI_GetNumTicksInRecord(PyObject *self, PyObject *args)
{
  struct ADI_FileHandle__ * x0;
  long x1;
  long * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  ADIResultCode result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ADI_GetNumTicksInRecord", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(25), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ADI_FileHandle__ *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(25), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, long);
  if (x1 == (long)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (long *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ADI_GetNumTicksInRecord(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ADI_GetNumTicksInRecord _cffi_d_ADI_GetNumTicksInRecord
#endif

static ADIResultCode _cffi_d_ADI_GetNumberOfChannels(struct ADI_FileHandle__ * x0, long * x1)
{
  return ADI_GetNumberOfChannels(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ADI_GetNumberOfChannels(PyObject *self, PyObject *args)
{
  struct ADI_FileHandle__ * x0;
  long * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  ADIResultCode result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ADI_GetNumberOfChannels", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(25), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ADI_FileHandle__ *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(25), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (long *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ADI_GetNumberOfChannels(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ADI_GetNumberOfChannels _cffi_d_ADI_GetNumberOfChannels
#endif

static ADIResultCode _cffi_d_ADI_GetNumberOfRecords(struct ADI_FileHandle__ * x0, long * x1)
{
  return ADI_GetNumberOfRecords(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ADI_GetNumberOfRecords(PyObject *self, PyObject *args)
{
  struct ADI_FileHandle__ * x0;
  long * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  ADIResultCode result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "ADI_GetNumberOfRecords", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(25), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ADI_FileHandle__ *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(25), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (long *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ADI_GetNumberOfRecords(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ADI_GetNumberOfRecords _cffi_d_ADI_GetNumberOfRecords
#endif

static ADIResultCode _cffi_d_ADI_GetRecordSamplePeriod(struct ADI_FileHandle__ * x0, long x1, long x2, double * x3)
{
  return ADI_GetRecordSamplePeriod(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ADI_GetRecordSamplePeriod(PyObject *self, PyObject *args)
{
  struct ADI_FileHandle__ * x0;
  long x1;
  long x2;
  double * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  ADIResultCode result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ADI_GetRecordSamplePeriod", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(25), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ADI_FileHandle__ *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(25), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, long);
  if (x1 == (long)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, long);
  if (x2 == (long)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(37), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(37), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ADI_GetRecordSamplePeriod(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ADI_GetRecordSamplePeriod _cffi_d_ADI_GetRecordSamplePeriod
#endif

static ADIResultCode _cffi_d_ADI_GetRecordTickPeriod(struct ADI_FileHandle__ * x0, long x1, long x2, double * x3)
{
  return ADI_GetRecordTickPeriod(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ADI_GetRecordTickPeriod(PyObject *self, PyObject *args)
{
  struct ADI_FileHandle__ * x0;
  long x1;
  long x2;
  double * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  ADIResultCode result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "ADI_GetRecordTickPeriod", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(25), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ADI_FileHandle__ *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(25), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, long);
  if (x1 == (long)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, long);
  if (x2 == (long)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(37), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(37), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ADI_GetRecordTickPeriod(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ADI_GetRecordTickPeriod _cffi_d_ADI_GetRecordTickPeriod
#endif

static ADIResultCode _cffi_d_ADI_GetRecordTime(struct ADI_FileHandle__ * x0, long x1, long * x2, double * x3, long * x4)
{
  return ADI_GetRecordTime(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ADI_GetRecordTime(PyObject *self, PyObject *args)
{
  struct ADI_FileHandle__ * x0;
  long x1;
  long * x2;
  double * x3;
  long * x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  ADIResultCode result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "ADI_GetRecordTime", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(25), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ADI_FileHandle__ *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(25), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, long);
  if (x1 == (long)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (long *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(37), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(37), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (long *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ADI_GetRecordTime(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ADI_GetRecordTime _cffi_d_ADI_GetRecordTime
#endif

static ADIResultCode _cffi_d_ADI_GetSamples(struct ADI_FileHandle__ * x0, long x1, long x2, long x3, ADICDataFlags x4, long x5, float * x6, long * x7)
{
  return ADI_GetSamples(x0, x1, x2, x3, x4, x5, x6, x7);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ADI_GetSamples(PyObject *self, PyObject *args)
{
  struct ADI_FileHandle__ * x0;
  long x1;
  long x2;
  long x3;
  ADICDataFlags x4;
  long x5;
  float * x6;
  long * x7;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  ADIResultCode result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;
  PyObject *arg7;

  if (!PyArg_UnpackTuple(args, "ADI_GetSamples", 8, 8, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6, &arg7))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(25), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ADI_FileHandle__ *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(25), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, long);
  if (x1 == (long)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, long);
  if (x2 == (long)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, long);
  if (x3 == (long)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x4, _cffi_type(57), arg4) < 0)
    return NULL;

  x5 = _cffi_to_c_int(arg5, long);
  if (x5 == (long)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(59), arg6, (char **)&x6);
  if (datasize != 0) {
    x6 = ((size_t)datasize) <= 640 ? (float *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(59), arg6, (char **)&x6,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg7, (char **)&x7);
  if (datasize != 0) {
    x7 = ((size_t)datasize) <= 640 ? (long *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg7, (char **)&x7,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ADI_GetSamples(x0, x1, x2, x3, x4, x5, x6, x7); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ADI_GetSamples _cffi_d_ADI_GetSamples
#endif

static ADIResultCode _cffi_d_ADI_GetUnitsName(struct ADI_FileHandle__ * x0, long x1, long x2, wchar_t * x3, long x4, long * x5)
{
  return ADI_GetUnitsName(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ADI_GetUnitsName(PyObject *self, PyObject *args)
{
  struct ADI_FileHandle__ * x0;
  long x1;
  long x2;
  wchar_t * x3;
  long x4;
  long * x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  ADIResultCode result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "ADI_GetUnitsName", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(25), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ADI_FileHandle__ *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(25), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, long);
  if (x1 == (long)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, long);
  if (x2 == (long)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(2), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (wchar_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(2), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x4 = _cffi_to_c_int(arg4, long);
  if (x4 == (long)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg5, (char **)&x5);
  if (datasize != 0) {
    x5 = ((size_t)datasize) <= 640 ? (long *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg5, (char **)&x5,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ADI_GetUnitsName(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ADI_GetUnitsName _cffi_d_ADI_GetUnitsName
#endif

static ADIResultCode _cffi_d_ADI_NextComment(struct ADI_CommentsHandle__ * x0)
{
  return ADI_NextComment(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ADI_NextComment(PyObject *self, PyObject *arg0)
{
  struct ADI_CommentsHandle__ * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  ADIResultCode result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(10), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct ADI_CommentsHandle__ *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(10), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ADI_NextComment(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ADI_NextComment _cffi_d_ADI_NextComment
#endif

static ADIResultCode _cffi_d_ADI_OpenFile(wchar_t const * x0, struct ADI_FileHandle__ * * x1, ADIFileOpenMode x2)
{
  return ADI_OpenFile(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_ADI_OpenFile(PyObject *self, PyObject *args)
{
  wchar_t const * x0;
  struct ADI_FileHandle__ * * x1;
  ADIFileOpenMode x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  ADIResultCode result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "ADI_OpenFile", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(83), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (wchar_t const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(83), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (struct ADI_FileHandle__ * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x2, _cffi_type(85), arg2) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = ADI_OpenFile(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_ADI_OpenFile _cffi_d_ADI_OpenFile
#endif

_CFFI_UNUSED_FN
static void _cffi_checkfld_struct_ADI_CommentsHandle__(struct ADI_CommentsHandle__ *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->unused) | 0);  /* check that 'struct ADI_CommentsHandle__.unused' is an integer */
}
struct _cffi_align_struct_ADI_CommentsHandle__ { char x; struct ADI_CommentsHandle__ y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld_struct_ADI_FileHandle__(struct ADI_FileHandle__ *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->unused) | 0);  /* check that 'struct ADI_FileHandle__.unused' is an integer */
}
struct _cffi_align_struct_ADI_FileHandle__ { char x; struct ADI_FileHandle__ y; };

static const struct _cffi_global_s _cffi_globals[] = {
  { "ADI_CloseCommentsAccessor", (void *)_cffi_f_ADI_CloseCommentsAccessor, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 6), (void *)_cffi_d_ADI_CloseCommentsAccessor },
  { "ADI_CloseFile", (void *)_cffi_f_ADI_CloseFile, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 21), (void *)_cffi_d_ADI_CloseFile },
  { "ADI_CreateCommentsAccessor", (void *)_cffi_f_ADI_CreateCommentsAccessor, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 70), (void *)_cffi_d_ADI_CreateCommentsAccessor },
  { "ADI_GetChannelName", (void *)_cffi_f_ADI_GetChannelName, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 75), (void *)_cffi_d_ADI_GetChannelName },
  { "ADI_GetCommentInfo", (void *)_cffi_f_ADI_GetCommentInfo, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 12), (void *)_cffi_d_ADI_GetCommentInfo },
  { "ADI_GetErrorMessage", (void *)_cffi_f_ADI_GetErrorMessage, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 0), (void *)_cffi_d_ADI_GetErrorMessage },
  { "ADI_GetNumSamplesInRecord", (void *)_cffi_f_ADI_GetNumSamplesInRecord, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 46), (void *)_cffi_d_ADI_GetNumSamplesInRecord },
  { "ADI_GetNumTicksInRecord", (void *)_cffi_f_ADI_GetNumTicksInRecord, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 28), (void *)_cffi_d_ADI_GetNumTicksInRecord },
  { "ADI_GetNumberOfChannels", (void *)_cffi_f_ADI_GetNumberOfChannels, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 24), (void *)_cffi_d_ADI_GetNumberOfChannels },
  { "ADI_GetNumberOfRecords", (void *)_cffi_f_ADI_GetNumberOfRecords, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 24), (void *)_cffi_d_ADI_GetNumberOfRecords },
  { "ADI_GetRecordSamplePeriod", (void *)_cffi_f_ADI_GetRecordSamplePeriod, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 40), (void *)_cffi_d_ADI_GetRecordSamplePeriod },
  { "ADI_GetRecordTickPeriod", (void *)_cffi_f_ADI_GetRecordTickPeriod, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 40), (void *)_cffi_d_ADI_GetRecordTickPeriod },
  { "ADI_GetRecordTime", (void *)_cffi_f_ADI_GetRecordTime, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 33), (void *)_cffi_d_ADI_GetRecordTime },
  { "ADI_GetSamples", (void *)_cffi_f_ADI_GetSamples, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 52), (void *)_cffi_d_ADI_GetSamples },
  { "ADI_GetUnitsName", (void *)_cffi_f_ADI_GetUnitsName, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 62), (void *)_cffi_d_ADI_GetUnitsName },
  { "ADI_NextComment", (void *)_cffi_f_ADI_NextComment, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 9), (void *)_cffi_d_ADI_NextComment },
  { "ADI_OpenFile", (void *)_cffi_f_ADI_OpenFile, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 82), (void *)_cffi_d_ADI_OpenFile },
  { "kADICDataAtSampleRate", (void *)_cffi_const_kADICDataAtSampleRate, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "kADICDataAtTickRate", (void *)_cffi_const_kADICDataAtTickRate, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "kOpenFileForReadAndWrite", (void *)_cffi_const_kOpenFileForReadAndWrite, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "kOpenFileForReadOnly", (void *)_cffi_const_kOpenFileForReadOnly, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "kResultADICAPIMsgBase", (void *)_cffi_const_kResultADICAPIMsgBase, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "kResultBufferTooSmall", (void *)_cffi_const_kResultBufferTooSmall, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "kResultErrorFlagBit", (void *)_cffi_const_kResultErrorFlagBit, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "kResultFail", (void *)_cffi_const_kResultFail, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "kResultFileIOError", (void *)_cffi_const_kResultFileIOError, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "kResultFileNotFound", (void *)_cffi_const_kResultFileNotFound, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "kResultFileOpenFail", (void *)_cffi_const_kResultFileOpenFail, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "kResultInvalidArg", (void *)_cffi_const_kResultInvalidArg, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "kResultInvalidCommentNum", (void *)_cffi_const_kResultInvalidCommentNum, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "kResultInvalidFileHandle", (void *)_cffi_const_kResultInvalidFileHandle, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "kResultInvalidPosition", (void *)_cffi_const_kResultInvalidPosition, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "kResultNoData", (void *)_cffi_const_kResultNoData, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "kResultSuccess", (void *)_cffi_const_kResultSuccess, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
};

static const struct _cffi_field_s _cffi_fields[] = {
  { "unused", offsetof(struct ADI_CommentsHandle__, unused),
              sizeof(((struct ADI_CommentsHandle__ *)0)->unused),
              _CFFI_OP(_CFFI_OP_NOOP, 89) },
  { "unused", offsetof(struct ADI_FileHandle__, unused),
              sizeof(((struct ADI_FileHandle__ *)0)->unused),
              _CFFI_OP(_CFFI_OP_NOOP, 89) },
};

static const struct _cffi_struct_union_s _cffi_struct_unions[] = {
  { "ADI_CommentsHandle__", 90, _CFFI_F_CHECK_FIELDS,
    sizeof(struct ADI_CommentsHandle__), offsetof(struct _cffi_align_struct_ADI_CommentsHandle__, y), 0, 1 },
  { "ADI_FileHandle__", 91, _CFFI_F_CHECK_FIELDS,
    sizeof(struct ADI_FileHandle__), offsetof(struct _cffi_align_struct_ADI_FileHandle__, y), 1, 1 },
};

static const struct _cffi_enum_s _cffi_enums[] = {
  { "ADICDataFlags", 57, _cffi_prim_int(sizeof(ADICDataFlags), ((ADICDataFlags)-1) <= 0),
    "kADICDataAtSampleRate,kADICDataAtTickRate" },
  { "ADIFileOpenMode", 85, _cffi_prim_int(sizeof(ADIFileOpenMode), ((ADIFileOpenMode)-1) <= 0),
    "kOpenFileForReadOnly,kOpenFileForReadAndWrite" },
  { "ADIResultCode", 1, _cffi_prim_int(sizeof(ADIResultCode), ((ADIResultCode)-1) <= 0),
    "kResultSuccess,kResultErrorFlagBit,kResultInvalidArg,kResultFail,kResultFileNotFound,kResultADICAPIMsgBase,kResultFileIOError,kResultFileOpenFail,kResultInvalidFileHandle,kResultInvalidPosition,kResultInvalidCommentNum,kResultNoData,kResultBufferTooSmall" },
};

static const struct _cffi_typename_s _cffi_typenames[] = {
  { "ADICDataFlags", 57 },
  { "ADIFileOpenMode", 85 },
  { "ADIResultCode", 1 },
  { "ADI_CommentsHandle", 10 },
  { "ADI_FileHandle", 25 },
  { "time_t", 3 },
};

static const struct _cffi_type_context_s _cffi_type_context = {
  _cffi_types,
  _cffi_globals,
  _cffi_fields,
  _cffi_struct_unions,
  _cffi_enums,
  _cffi_typenames,
  34,  /* num_globals */
  2,  /* num_struct_unions */
  3,  /* num_enums */
  6,  /* num_typenames */
  NULL,  /* no includes */
  93,  /* num_types */
  0,  /* flags */
};

#ifdef __GNUC__
#  pragma GCC visibility push(default)  /* for -fvisibility= */
#endif

#ifdef PYPY_VERSION
PyMODINIT_FUNC
_cffi_pypyinit__adi_cffi(const void *p[])
{
    p[0] = (const void *)0x2601;
    p[1] = &_cffi_type_context;
#if PY_MAJOR_VERSION >= 3
    return NULL;
#endif
}
#  ifdef _MSC_VER
     PyMODINIT_FUNC
#  if PY_MAJOR_VERSION >= 3
     PyInit__adi_cffi(void) { return NULL; }
#  else
     init_adi_cffi(void) { }
#  endif
#  endif
#elif PY_MAJOR_VERSION >= 3
PyMODINIT_FUNC
PyInit__adi_cffi(void)
{
  return _cffi_init("_adi_cffi", 0x2601, &_cffi_type_context);
}
#else
PyMODINIT_FUNC
init_adi_cffi(void)
{
  _cffi_init("_adi_cffi", 0x2601, &_cffi_type_context);
}
#endif

#ifdef __GNUC__
#  pragma GCC visibility pop
#endif
